# Author: ESRI (#5588)
# Date:   February 5th 2009
# Purpose: This script demonstrates a methodology for very large batch geocoding.
#          The scenario is the source table is a DBMS view in a remote DBMS accessed
#          via an OLE DB connection, and the destination feature class is truncated
#          at each run - we are maintaining a spatial edition of the DBMS table.
#          
#
try:
    import arcgisscripting, os.path, time, sys, traceback
    gp = arcgisscripting.create(9.3)
    gp.overwriteoutput = 1

    #Record the start time
    startTime = time.clock()
    gp.addmessage("Started batch geocoding job at: " + time.ctime())

    #Create a dictionary with GP result status explanations
    statusDict = {0:"New",1:"Submitted",2:"Waiting",3:"Executing",4:"Succeeded",5:"Failed", \
                  6:"Timed Out",7:"Canceling",8:"Canceled",9:"Deleting",10:"Deleted"}    

    #We will append the results to an existing template feature class; truncate it first
    outFC = r"C:\arcgisserver\Toolshare\Tooldata\Output.gdb\Customers"
    #outFC = r"R:\Output.gdb\Customers"
    gp.addmessage("Truncating output feature class: " + outFC)
    truncResult = gp.deleterows_management(outFC)
    truncResult = gp.getcount_management(outFC)
    gp.addmessage("Truncation result is: " +str(truncResult.getoutput(0)) + " rows")

    #This script is in the Toolshare structure, go pick up the connection and input table
    #toolSharePath = os.path.dirname(sys.path[0])
    toolSharePath = r"C:\arcgisserver\Toolshare"
    gp.addmessage("Toolshare path is: " + toolSharePath)
    oledbWorkspace = toolSharePath + r"\Connections\AGSUSER on BRUCEH.odc"
    #The fully qualified table name will be source DBMS dependent - edit this next line appropriately
    #Also ensure the table you reference agrees with the data source in the GP model
    inputTable = oledbWorkspace + r"\dbo.FiveMillion"
    gp.addmessage("Reading table: " + gp.parsetablename(inputTable).split(",")[2])
    
    #Get the part count desired.  This will be a multiple of available CPU cores - recommended multiple is 1
    #partCount = 8
    partCount = int(gp.getparameter(0))
    gp.addmessage("Splitting input table into " + str(partCount) + " parts.")

    #Count the input table rows
    result = gp.getcount_management(inputTable)
    count = int(result.getoutput(0))
    gp.addmessage("Table " + inputTable + " has rowcount " + str(count))

    #Our GP service makes a query table on the ID field, so the data mustbe appropriately structured
    #Calculate the intervals suitable for a query ID >= <n> AND ID < <n>
    intervals = range(1,count,int(count/partCount))
    if len(intervals) > partCount:
        intervals[-1] = count + 1
    else:
        intervals.append(count+1)

    #Create the queries for each table part.
    #The GP service will copy rows into the scratch workspace so delimit appropriately
    dFName = gp.addfielddelimiters(gp.scratchworkspace,"ID")
    queryList = []
    for i in range(len(intervals)-1):
        queryList.append(dFName + " >= " + str(intervals[i]) + " AND " \
                         + dFName + " < " + str(intervals[i+1]))
        gp.addmessage("Built query: " + queryList[i])
    

    #Send each table part to the geocoding service
    resultList, appendedList = [],[]
    gp.addtoolbox("http://eslsrv23/arcgis/services;GeocodingBestPractice/ServerGeocode")
    for q in queryList:
        gp.addmessage("Sending query " + str(q) + " to the geocode service")
        result = gp.GeocodeCustomers_ServerGeocode(str(q))
        resultList.append(result)


    #Wait until all the query geocoding results have begun turning up
    numGeocoding = 0
    resultPrefix = r"C:\arcgisserver\arcgisjobs\geocodingbestpractice\servergeocode_gpserver" + "\\"
    resultSuffix = r"\scratch\scratch.gdb\Geocode_Result"
    gp.addmessage("\n")
    while (numGeocoding < partCount) and (time.clock() - startTime < 360):
        gp.addmessage("Waiting for all geocoding processes to start..")
        numGeocoding = 0
        time.sleep(20)
        for r in resultList:
            resultPath = resultPrefix + r.resultid + resultSuffix
            resultExists = gp.exists(resultPath)
            gp.addmessage("Result " + str(r.resultID) + " exists?:" + str(resultExists))
            if resultExists:
                numGeocoding = numGeocoding + 1
        gp.addmessage(str(numGeocoding) + " of " + str(partCount) + " parts now geocoding\n")
            

    
    #Monitor geocode results as they turn up then append them to the target feature class
    endStatus = 0
    while endStatus < 4:
        loopStart = time.clock()
        statusList = []
        resultPath = ""
        #For each part completing in this interval, get the result path
        for r in resultList:
            thisStatus = r.status
            statusList.append(thisStatus)
            gp.addmessage("Result "+ str(r.resultID) + " has status \'" + statusDict[thisStatus] + "\'")
            if thisStatus == 4 and appendedList.count(r.resultid) == 0:
                gp.addmessage("Result " + str(r.resultID) + " ready to append")
                resultPath = resultPath + resultPrefix + str(r.resultID) + resultSuffix + ";"
                appendedList.append(r.resultID)
        #Append anything that has completed.  Remove the spatial index for speed.  Append will restore it.
        if len(resultPath) <> 0:
            resultPath = resultPath.rstrip(";")
            gp.addmessage("\nAppending from " + resultPath)
            outDesc = gp.describe(outFC)
            if outDesc.hasspatialindex:
                gp.removespatialindex_management(outFC)
            appendResult = gp.append_management(resultPath,outFC,"NO_TEST")
            gp.addmessage("Append complete\n")
        else:
            loopEnd = time.clock()
            loopDuration = int(loopEnd - loopStart)
            if (loopDuration) < 60:
                gp.addmessage(str(time.ctime()) + " Waiting balance of 60 seconds: = " + str(60 - loopDuration) + " sec.\n")
                time.sleep(60 - loopDuration)
        statusList.sort()
        endStatus = statusList[0]

    gp.addmessage  ("\nLast geocode/append process complete.")
    appendResult = gp.getcount_management(outFC)
    gp.addmessage("Output feature class has: " +str(appendResult.getoutput(0)) + " rows")
   
    #Because this script runs on the server we can pick up the results off disk and not return
    #them to the script (code for the more orthodox approach is commented out)
    #vTab = gp.createobject("Valuetable",1)
    #for r in range(partCount):
    #    vTab.addrow(eval("result" + str(r)+".getoutput(0)"))
    #gp.addmessage("Appending results...")
    #gp.append_management(vTab.exporttostring(),outFC,"NO_TEST")
    #p.addmessage("Appending results...")
    #print "Appending results..."
    #resultPath = r""
    #for r in range(partCount):
    #    resultPath = resultPath + r"C:\arcgisserver\arcgisjobs\geocodingbestpractice\servergeocode_gpserver"
    #    resultPath = resultPath + "\\" + eval("str(result"+str(r)+".resultid)")
    #    resultPath = resultPath + r"\scratch\scratch.gdb\Geocode_Result;"
    #    resultPath = resultPath.rstrip(";")
    #gp.addmessage("Result path is: " + resultPath)
    #print "Result path is: " + resultPath
    #appendResult = gp.append_management(resultPath,outFC,"NO_TEST")
    #gp.addmessage("Append Complete")
    #print "Append Complete"

    endTime = time.clock()
    gp.addmessage("Finished batch geocoding job at: " + time.ctime())
    gp.addmessage("Net performance was " + \
                  str(int(1.0 * count / (endTime - startTime) * 3600.0)) + " records per hour.")
    gp = None
except:
    tb = sys.exc_info()[2]
    tbinfo = traceback.format_tb(tb)[0]
    pymsg = "PYTHON ERRORS:\nTraceback Info:\n" + tbinfo + "\nError Info:\n    " + \
            str(sys.exc_type)+ ": " + str(sys.exc_value) + "\n"
    print pymsg
    gp.AddError(pymsg)
    
    msgs = "GP ERRORS:\n" + gp.GetMessages(2) + "\n"
    print msgs
    gp.AddError(msgs)