# Author: ESRI (#5588)
# Date:   February 26th 2009
# Purpose: This script reads a table and writes a copy of the table sorted
#          in ascending order on a set of fields chosen by the user.  The
#          tool is intended as a preprocess to geocoding, with the sorting
#          designed to suit a locator index SortyBy structure
#
try:
    import arcgisscripting, os.path, string, sys, traceback
    gp = arcgisscripting.create(9.3)
    gp.overwriteoutput = 1

    #Get the source table
    inTable = gp.getparameterastext(0)
    gp.addmessage("\nReading table '" + os.path.basename(inTable) + "'")
    inDesc = gp.describe(inTable)

    #Count the table rows
    result = gp.getcount_management(inTable)
    count = int(result.getoutput(0))
    gp.addmessage("\nTable has rowcount: " + str(count))

    #Get the sort fields valuetable into a list of fields
    userFieldVTab = gp.getparameter(1)
    sortFieldList = []
    for f in range(0,userFieldVTab.rowcount):
        sortFieldList.append(userFieldVTab.getvalue(f,0))

    #Get the sort ranks valuetable and re-order the sort fields accordingly
    #Tool validation ensures the sort ranks will work with the sort fields
    fieldRankVTab = gp.getparameter(2)
    fieldRankList = []
    for r in range(0,fieldRankVTab.rowcount):
        fieldRankList.append(int(fieldRankVTab.getvalue(r,0)))
    rankedFieldList = []
    sortMsg = "\nSorting on "
    for r in range(0,len(fieldRankList)):
        rankField = sortFieldList[fieldRankList.index(r)]
        rankedFieldList.append(rankField)
        sortMsg = sortMsg + str(rankField) + ","
    sortFieldList = rankedFieldList
    sortMsg = sortMsg.rstrip(",")
    gp.addmessage(sortMsg)

    #Use the sort fields to make the cursor sortFields argument
    sortFields = ""
    for f in sortFieldList:
        sortFields = sortFields + str(f) + " A;"
    sortFields = sortFields.rstrip(";")    

    #Create the output table with the source table as template
    inWorkspace = os.path.dirname(inTable)
    outTable = gp.getparameterastext(3)
    if outTable == "":
        outTable = os.path.basename(inTable)+"_sorted"
        gp.workspace = inWorkspace
        gp.addmessage("\nOutput table will be " + str(outTable))
    else:
        gp.workspace = os.path.dirname(outTable)
        outTable = os.path.basename(outTable)
    gp.createtable_management(gp.Workspace,outTable,inTable)

    #Open cursors on the source and target tables, the source cursor is sorted
    gp.addmessage("\nSorting table '" + os.path.basename(inTable) + "' to create '" + outTable + "'")
    inCur = gp.searchcursor(inTable,"","","",sortFields)
    #Get a list of non-OID fields
    writeFields = []
    inFields = gp.listfields(inTable)
    if inDesc.HasOID:
        for f in inFields:
            if f.Name <> inDesc.OIDFieldName:
                writeFields.append(f)
    else:
        writeFields = inFields

    gp.setprogressor("step","Writing to " + outTable,0,100,1)
    outCur = gp.insertcursor(outTable)
    inRow = inCur.Next()
    i = 1.0
    percent = 0
    while inRow:
        row = outCur.NewRow()
        for ff in writeFields:
            row.setvalue(ff.Name,inRow.getvalue(ff.name))
        outCur.Insertrow(row)
        inRow = inCur.Next()
        i = i + 1.0
        progress = int(i / count * 100.0)
        if progress > percent:
            percent = int(progress)
            gp.setprogressorlabel(str(percent) + "% complete")
            gp.setprogressorposition(percent)
    gp.resetprogressor()
    del inCur
    del outCur
    del gp
    #
except:
    tb = sys.exc_info()[2]
    tbinfo = traceback.format_tb(tb)[0]
    pymsg = "PYTHON ERRORS:\nTraceback Info:\n" + tbinfo + "\nError Info:\n    " + \
            str(sys.exc_type)+ ": " + str(sys.exc_value) + "\n"
    print pymsg
    gp.AddError(pymsg)
    
    msgs = "GP ERRORS:\n" + gp.GetMessages(2) + "\n"
    print msgs
    gp.AddError(msgs)